Package: testpackage
Version: 1.0.0
Title: A test package
Description: A longer statement about the package.
Authors@R:
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"))
RoxygenNote: 6.0.1
