# Generated by roxygen2: do not edit by hand

S3method("[",tag)
S3method(as_data,"NULL")
S3method(as_data,tag_arguments)
S3method(as_data,tag_author)
S3method(as_data,tag_description)
S3method(as_data,tag_details)
S3method(as_data,tag_format)
S3method(as_data,tag_note)
S3method(as_data,tag_references)
S3method(as_data,tag_section)
S3method(as_data,tag_seealso)
S3method(as_data,tag_source)
S3method(as_data,tag_usage)
S3method(as_data,tag_value)
S3method(as_example,COMMENT)
S3method(as_example,RCODE)
S3method(as_example,TEXT)
S3method(as_example,VERB)
S3method(as_example,tag)
S3method(as_example,tag_dontrun)
S3method(as_example,tag_dontshow)
S3method(as_example,tag_donttest)
S3method(as_example,tag_dots)
S3method(as_example,tag_if)
S3method(as_example,tag_ifelse)
S3method(as_example,tag_ldots)
S3method(as_example,tag_out)
S3method(as_example,tag_testonly)
S3method(as_html,"#ifdef")
S3method(as_html,"#ifndef")
S3method(as_html,COMMENT)
S3method(as_html,LIST)
S3method(as_html,RCODE)
S3method(as_html,Rd)
S3method(as_html,TEXT)
S3method(as_html,USERMACRO)
S3method(as_html,VERB)
S3method(as_html,character)
S3method(as_html,tag)
S3method(as_html,tag_R)
S3method(as_html,tag_S3method)
S3method(as_html,tag_S4method)
S3method(as_html,tag_Sexpr)
S3method(as_html,tag_acronym)
S3method(as_html,tag_bold)
S3method(as_html,tag_cite)
S3method(as_html,tag_code)
S3method(as_html,tag_command)
S3method(as_html,tag_cr)
S3method(as_html,tag_dQuote)
S3method(as_html,tag_deqn)
S3method(as_html,tag_describe)
S3method(as_html,tag_dfn)
S3method(as_html,tag_dots)
S3method(as_html,tag_email)
S3method(as_html,tag_emph)
S3method(as_html,tag_enc)
S3method(as_html,tag_enumerate)
S3method(as_html,tag_env)
S3method(as_html,tag_eqn)
S3method(as_html,tag_figure)
S3method(as_html,tag_file)
S3method(as_html,tag_href)
S3method(as_html,tag_if)
S3method(as_html,tag_ifelse)
S3method(as_html,tag_item)
S3method(as_html,tag_itemize)
S3method(as_html,tag_kbd)
S3method(as_html,tag_ldots)
S3method(as_html,tag_link)
S3method(as_html,tag_linkS4class)
S3method(as_html,tag_method)
S3method(as_html,tag_newcommand)
S3method(as_html,tag_option)
S3method(as_html,tag_out)
S3method(as_html,tag_pkg)
S3method(as_html,tag_preformatted)
S3method(as_html,tag_renewcommand)
S3method(as_html,tag_sQuote)
S3method(as_html,tag_samp)
S3method(as_html,tag_special)
S3method(as_html,tag_strong)
S3method(as_html,tag_subsection)
S3method(as_html,tag_tab)
S3method(as_html,tag_tabular)
S3method(as_html,tag_url)
S3method(as_html,tag_var)
S3method(as_html,tag_verb)
S3method(pkgdown_print,default)
S3method(pkgdown_print,gt_tbl)
S3method(pkgdown_print,htmlwidget)
S3method(print,pkgdown_xml)
S3method(print,print_yaml)
S3method(print,tag)
S3method(usage_code,"NULL")
S3method(usage_code,COMMENT)
S3method(usage_code,RCODE)
S3method(usage_code,Rd)
S3method(usage_code,TEXT)
S3method(usage_code,VERB)
S3method(usage_code,tag)
S3method(usage_code,tag_S3method)
S3method(usage_code,tag_S4method)
S3method(usage_code,tag_dots)
S3method(usage_code,tag_ldots)
S3method(usage_code,tag_method)
S3method(usage_code,tag_special)
S3method(usage_code,tag_usage)
export(as_pkgdown)
export(build_article)
export(build_articles)
export(build_articles_index)
export(build_favicons)
export(build_home)
export(build_home_index)
export(build_llm_docs)
export(build_news)
export(build_redirects)
export(build_reference)
export(build_reference_index)
export(build_search)
export(build_site)
export(build_site_github_pages)
export(build_tutorials)
export(check_pkgdown)
export(clean_cache)
export(clean_site)
export(data_template)
export(deploy_site_github)
export(deploy_to_branch)
export(fig_settings)
export(in_pkgdown)
export(init_site)
export(pkgdown_print)
export(pkgdown_sitrep)
export(preview_site)
export(rd2html)
export(render_page)
export(template_articles)
export(template_navbar)
export(template_reference)
import(fs)
import(rlang)
importFrom(lifecycle,deprecated)
importFrom(utils,installed.packages)
