# Authors

By default, pkgdown will display author information in three places:

* the sidebar,
* the left part side of the footer,
* the author page.

This documentation describes how to customise the overall author display. See `?build_home` and `?build_site` for details about changing the location of the authors information within the home sidebar and the site footer.

## Authors ORCID and bio

Author ORCID identification numbers in the `DESCRIPTION` are linked using the ORCID logo:

```r
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"),
      comment = c(ORCID = "0000-0003-4757-117X")
    ),
    person("Jay", "Hesselberth", role = "aut",
      comment = c(ORCID = "0000-0002-6299-179X")
    )
  )
```

If you want to add more details about authors or their involvement with the package,
you can use the comment field, which will be rendered on the authors page.

```r
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre"),
      comment = c(ORCID = "0000-0003-4757-117X", "Indenter-in-chief")
    ),
    person("Jay", "Hesselberth", role = "aut",
      comment = c(ORCID = "0000-0002-6299-179X")
    )
  )
```

## YAML config - authors

You can tweak a few things via the `authors` YAML field:

* display of each author in the footer, sidebar and authors page,
* which authors (by role) are displayed in the sidebar and footer,
* text before authors in the footer, 
* text before and after authors in the sidebar,
* text before and after authors on the authors page.


You can modify how each author's name is displayed
by adding a subsection for `authors`. 
Each entry in `authors` should be named 
with the author's name (matching `DESCRIPTION`)
and can contain `href` and/or `html` fields:

* If `href` is provided, the author's name will be linked to this URL.
* If `html` is provided, it will be shown instead of the author's name.
  This is particularly useful if you want to display the logo of a corporate
  sponsor. Use an absolute URL to an image, not a relative link.

```yaml
authors:
  firstname lastname:
    href: "http://name-website.com"
    html: "<img src='https://website.com/name-picture.png' width=72>"
```


By default, the "developers" list shown in the sidebar and footer is populated by the maintainer ("cre"), authors
("aut"), and funder ("fnd") from the `DESCRIPTION`. 
You could choose other roles for filtering.
With the configuration below:

* only the maintainer and funder(s) appear in the footer, after the text "Crafted by",
* all authors and contributors appear in the sidebar,
* the authors list on the sidebar is preceded and followed by some text,
* the authors list on the authors page is preceded and followed by some text.


```yaml
authors:
  footer:
    roles: [cre, fnd]
    text: "Crafted by"
  sidebar:
    roles: [aut, ctb]
    before: "So *who* does the work?"
    after: "Thanks all!"
  before: "This package is proudly brought to you by:"
  after: "See the [changelog](news/index.html) for other contributors. :pray:"
```

If you want to filter authors based on something else than their roles, 
consider using a custom sidebar/footer component 
(see `?build_home`/`?build_site`, respectively).
