---
title: "html_document + as_is"
output: 
  html_document:
    number_sections: true
pkgdown:
  as_is: true
---

```{r include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This should have section numbers on the headings.

# Heading 1

# Heading 2
