# we find out if quarto styles change

    Code
      cat(data$includes$style)
    Output
      
          code{white-space: pre-wrap;}
          span.smallcaps{font-variant: small-caps;}
          div.columns{display: flex; gap: min(4vw, 1.5em);}
          div.column{flex: auto; overflow-x: auto;}
          div.hanging-indent{margin-left: 1.5em; text-indent: -1.5em;}
          ul.task-list{list-style: none;}
          ul.task-list li input[type="checkbox"] {
            width: 0.8em;
            margin: 0 0.8em 0.2em -1em; /* quarto-specific, see https://github.com/quarto-dev/quarto-cli/issues/4556 */ 
            vertical-align: middle;
          }
        

