---
title: "HTML Deps"
---

```{r}
# Call some code that adds jquery/bootstrap
htmltools::tagList(
  htmltools::p("hello"),
  rmarkdown::html_dependency_jquery(),
  rmarkdown::html_dependency_bootstrap("flatly")
)
```


```{r}
knitr::knit_hooks$set(document = function(x) {
  if (interactive()) {
    browser()
  }
  x
})
```
