# informative print method

    Code
      init_site(pkg)
    Message
      -- Initialising site -----------------------------------------------------------
      Copying <pkgdown>/BS5/assets/katex-auto.js to katex-auto.js
      Copying <pkgdown>/BS5/assets/lightswitch.js to lightswitch.js
      Copying <pkgdown>/BS5/assets/link.svg to link.svg
      Copying <pkgdown>/BS5/assets/pkgdown.js to pkgdown.js
      Updating deps/bootstrap-5.3.1/bootstrap.bundle.min.js
      Updating deps/bootstrap-5.3.1/bootstrap.bundle.min.js.map
      Updating deps/bootstrap-5.3.1/bootstrap.min.css
      Updating deps/bootstrap-toc-1.0.1/bootstrap-toc.min.js
      Updating deps/clipboard.js-2.0.11/clipboard.min.js
      Updating deps/font-awesome-6.4.2/css/all.css
      Updating deps/font-awesome-6.4.2/css/all.min.css
      Updating deps/font-awesome-6.4.2/css/v4-shims.css
      Updating deps/font-awesome-6.4.2/css/v4-shims.min.css
      Updating deps/font-awesome-6.4.2/webfonts/fa-brands-400.ttf
      Updating deps/font-awesome-6.4.2/webfonts/fa-brands-400.woff2
      Updating deps/font-awesome-6.4.2/webfonts/fa-regular-400.ttf
      Updating deps/font-awesome-6.4.2/webfonts/fa-regular-400.woff2
      Updating deps/font-awesome-6.4.2/webfonts/fa-solid-900.ttf
      Updating deps/font-awesome-6.4.2/webfonts/fa-solid-900.woff2
      Updating deps/font-awesome-6.4.2/webfonts/fa-v4compatibility.ttf
      Updating deps/font-awesome-6.4.2/webfonts/fa-v4compatibility.woff2
      Updating deps/headroom-0.11.0/headroom.min.js
      Updating deps/headroom-0.11.0/jQuery.headroom.min.js
      Updating deps/jquery-3.6.0/jquery-3.6.0.js
      Updating deps/jquery-3.6.0/jquery-3.6.0.min.js
      Updating deps/jquery-3.6.0/jquery-3.6.0.min.map
      Updating deps/search-1.0.0/autocomplete.jquery.min.js
      Updating deps/search-1.0.0/fuse.min.js
      Updating deps/search-1.0.0/mark.min.js

# site meta doesn't break unexpectedly

    Code
      yaml
    Output
      pandoc: '{version}'
      pkgdown: '{version}'
      pkgdown_sha: '{sha}'
      articles:
        a: a.html
        b: b.html
      last_built: 2020-01-01T00:00Z
      

