% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test-figures}
\alias{test-figures}
\title{Test case: figures}
\description{
\figure{bacon.jpg}
}
\examples{
x <- seq(0, 2 * pi, length.out = 25)
plot(x, sin(x))

plot(1:10)
lines(1:10)
text(2, 5, "Hello", srt = 30, cex = 2)

plot(1:3, col = c("red", "green", "blue"), pch = 16, cex = 10)
}
\seealso{
Other tests: 
\code{\link{index}},
\code{\link{test-crayon}},
\code{\link{test-dont}},
\code{\link{test-links}},
\code{\link{test-lists}},
\code{\link{test-long-lines}},
\code{\link{test-math-examples}},
\code{\link{test-output-styles}},
\code{\link{test-params}},
\code{\link{test-sexpr-title}},
\code{\link{test-tables}},
\code{\link{test-verbatim}}
}
\concept{tests}
\keyword{internal}
