% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{clean_site}
\alias{clean_site}
\title{Clean site}
\usage{
clean_site(pkg = ".")
}
\arguments{
\item{pkg}{Path to package.}
}
\description{
Delete all files in \verb{docs/} (except for \code{CNAME}).
}
