% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bacon.R
\name{Figure}
\alias{Figure}
\title{Figure}
\description{
Pulled pork is delicious
}
\examples{
plot(1:5)
hist(1:3)
}
\keyword{internal}
