% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funs.R
\name{a}
\alias{a}
\title{A}
\usage{
a(a, b, c)
}
\arguments{
\item{a}{a letter}

\item{b}{a a number}

\item{c}{a logical}
}
\description{
A
}
\keyword{foo}
