% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funs.R
\name{a}
\alias{a}
\title{Example with HTML dependency}
\usage{
a()
}
\description{
Example with HTML dependency
}
\examples{
a()
}
